# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2010 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import itertools
import inspect

import unittest

import asynchia.ee
import asynchia.dsl

from asynchia.util import b
from asynchia.dsl import LFLSE, lookback, FLSE
from asynchia.dsl import s as db

StringInput = None

try:
    from inspect import isgenerator
except ImportError:
    from types import GeneratorType
    def isgenerator(obj):
        return isinstance(obj, GeneratorType)


def exhaust(itr):
    result = []
    for elem in itr:
        if isgenerator(elem):
            result.append(exhaust(iter(elem)))
        else:
            result.append(elem)
    return result


def until_done(fun):
    while True:
        d, s = fun()
        if d:
            break
    

class TestDSL(unittest.TestCase):
    def setUp(self):
        global StringInput
        StringInput = asynchia.ee.StringInput 
        asynchia.ee.StringInput = lambda x: x
    
    
    def tearDown(self):
        asynchia.ee.StringInput = StringInput
    
    def test_LFLSE(self):
        e = db.L() + db.B() + LFLSE(-1)
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        a = e(None)
        until_done(lambda: a.add_data(m, 120))
        
        self.assertEqual(tuple(a.value), (5, 1, b('A')))
    
        
    def test_two_instances(self):
        e = db.L() + db.B() + LFLSE(-1)
        a = e()
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: a.add_data(m, 120))
        
        self.assertEqual(tuple(a.value), (5, 1, b('A')))
        c = e()
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: c.add_data(m, 120))
        
        self.assertEqual(tuple(a.value), (5, 1, b('A')))
    
    
    def test_example(self):
        e = db.L() + db.B() + LFLSE(0)
        a = e(None)
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: a.add_data(m, 120))
        
        self.assertEqual(tuple(a.value), (5, 1, b('ABCDE')))
    
    
    def test_nested(self):
        i = [2, b('AB'), [5, b('ABCDE')], [5, b('ABCDE')]]
        
        a = db.B() + LFLSE(0)
        c = db.B() + LFLSE(0) + a + a
        
        d = c.produce(i)
        
        p = c(None)
        
        m = asynchia.ee.MockHandler(inbuf=d + b('FG'))
        until_done(lambda: p.add_data(m, 120))
        
        self.assertEqual(exhaust(iter(p.value)), i)
    
    
    def test_named(self):
        e = db.L()['size'] + db.B()['blub'] + FLSE(lookback('size'))['string']
        
        a = e(None)
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: a.add_data(m, 120))
        
        self.assertEqual(tuple(a.value), (5, 1, b('ABCDE')))
        self.assertEqual(m.inbuf, b('FG'))
    
    
    def test_tonamed(self):
        e = db.L()['size'] + db.B()['blub'] + FLSE(lookback('size'))['string']
        
        a = e(None)
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: a.add_data(m, 120))
        
        d = e.tonamed(a.value)
        self.assertEqual(d['size'], 5)
        self.assertEqual(d['blub'], 1)
        self.assertEqual(d['string'], b('ABCDE'))
    
    
    def test_tonamed2(self):
        e = db.L()['size'] + db.L()['blub'] + FLSE(lookback('size'))['string']
        
        a = e(None)
        m = asynchia.ee.MockHandler(
            inbuf=e.produce((5, 1, b('ABCDE'))) + b('FG')
        )
        until_done(lambda: a.add_data(m, 120))
        
        d = e.tonamed(a.value)
        self.assertEqual(d['size'], 5)
        self.assertEqual(d['blub'], 1)
        self.assertEqual(d['string'], b('ABCDE'))

if __name__ == '__main__':
    unittest.main()



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement

import time
import errno
import socket
import threading

import asynchia
import asynchia.maps
import asynchia.util

import asynchia.forthcoming

b = asynchia.util.b

import unittest

def _override_socketpair(fun):
    def _fun(*args, **kwargs):
        bu = getattr(socket, 'socketpair', None)
        del socket.socketpair
        try:
            return fun(*args, **kwargs)
        finally:
            if bu is not None:
                socket.socketpair = bu
    return _fun


class Container(object):
    pass


def dnr_forthcoming_wakeup(self, map_):
    container = Container()
    container.flag = False
    
    mo = map_()
    
    mainthread = threading.currentThread()
    
    def thr(nf):
        time.sleep(2)
        nf.inject(12)
    
    def callb(data):
        self.assertEquals(data, 12)
        self.assertEquals(threading.currentThread(), mainthread)
        
        container.flag = True
    
    nf = asynchia.forthcoming.DataNotifier(mo)
    nf.add_databack(callb)
    th = threading.Thread(target=thr, args=(nf,))
    th.start()
    
    s = time.time()
    while not container.flag and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEquals(container.flag, True)


def dnr_interrupt(self, map_):
    container = Container()
    container.flag = False
    mo = map_()
    def thread(container):
        mo.start_interrupt()
        try:
            time.sleep(4)
        finally:
            container.flag = True
            mo.end_interrupt()
    threading.Thread(target=thread, args=(container, )).start()
    s = time.time()
    while not container.flag and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEqual(container.flag, True)


cf_done = False
cf_thr = None

        
def std(mo, hand):
    time.sleep(1)
    mo.start_interrupt(True)
    try:
        hand.set_writeable(True)
    finally:
        mo.end_interrupt(True)


def ctx(mo, hand):
    time.sleep(1)
    with mo.interrupt(True):
        hand.set_writeable(True)


def t_changeflag(subthread):
    def dnr_changeflag(self, map_):
        container = Container()
        container.done = False
        container.thr = None        
        
        class Serv(asynchia.Server):
            def __init__(
                self, transport, handlercls=asynchia.Handler
                ):
                asynchia.Server.__init__(self, transport, handlercls)
                self.clients = []
            
            def new_connection(self, handler, addr):
                self.clients.append(handler)
        
        
        class Handler(asynchia.Handler):
            def __init__(self, transport, container=None):
                asynchia.Handler.__init__(self, transport)
                self.container = container
            
            def handle_connect(self):
                container.thr = threading.Thread(
                    target=subthread,
                    args=[self.transport.socket_map, self.transport]
                )
                container.thr.start()
            
            def handle_write(self):
                container.done = True
                self.transport.set_writeable(False)
            
            # Prevent exception from being suppressed.
            def handle_error(self):
                raise
        
        mo = map_()
        s = Serv(asynchia.SocketTransport(mo))
        s.transport.bind(('127.0.0.1', 0))
        # We know we'll only get one connection.
        s.transport.listen(1)
        
        c = Handler(asynchia.SocketTransport(mo), container)
        c.transport.connect(s.transport.socket.getsockname())
        n = 0
        s = time.time()
        while not container.done and time.time() < s + 10:
            mo.poll(abs(10 - (time.time() - s)))
        self.assertEqual(container.done, True)
        mo.close()
        container.thr.join(10)
        self.assertEqual(container.thr.isAlive(), False)
    return dnr_changeflag
    
def dnr_remove(self, map_):
    mo = map_()
    container = Container()
    container.done = False
    
    class Serv(asynchia.Server):
        def __init__(
            self, transport, handlercls=asynchia.Handler
            ):
            asynchia.Server.__init__(self, transport, handlercls)
            self.clients = []
        
        def new_connection(self, handler, addr):
            self.clients.append(handler)
    
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
            self.transport.set_writeable(True)
        
        def handle_write(self):
            container.done = True
            
        # Prevent exception from being suppressed.
        def handle_error(self):
            raise
    
    mo = map_()
    s = Serv(asynchia.SocketTransport(mo))
    s.transport.bind(('127.0.0.1', 0))
    # We know we'll only get one connection.
    s.transport.listen(1)
    
    c = Handler(asynchia.SocketTransport(mo), container)
    c.transport.connect(s.transport.socket.getsockname())
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEqual(container.done, True)
    mo.del_transport(c.transport)
    container.done = False
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    mo.close()
    self.assertEqual(container.done, False)


def dnr_remove2(self, map_):
    mo = map_()
    container = Container()
    container.done = False
    
    class Serv(asynchia.Server):
        def __init__(
            self, transport, handlercls=asynchia.Handler
            ):
            asynchia.Server.__init__(self, transport, handlercls)
            self.clients = []
        
        def new_connection(self, handler, addr):
            self.clients.append(handler)
    
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
        
        def handle_connect(self):
            self.transport.set_writeable(True)
        
        def handle_write(self):
            container.done = True
            
        # Prevent exception from being suppressed.
        def handle_error(self):
            raise
    
    mo = map_()
    s = Serv(asynchia.SocketTransport(mo))
    s.transport.bind(('127.0.0.1', 0))
    # We know we'll only get one connection.
    s.transport.listen(1)
    
    c = Handler(asynchia.SocketTransport(mo), container)
    c.transport.connect(s.transport.socket.getsockname())
    s = time.time()
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEqual(container.done,  True)
    mo.del_transport(c.transport)
    container.done = False
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    mo.close()
    self.assertEqual(container.done, False)


def dnr_close(self, map_):
    mo = map_()
    container = Container()
    container.done = False
    
    a, b = asynchia.util.socketpair()
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
        
        def handle_close(self):
            container.done = True
    
    
    mo = map_()
    
    c = Handler(asynchia.SocketTransport(mo, a), container)
    b.close()
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    mo.close()
    self.assertEqual(container.done, True)


def dnr_close_read(self, map_):
    mo = map_()
    container = Container()
    container.done = False
    
    a, b = asynchia.util.socketpair()
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
            
            self.transport.set_readable(True)
        
        def handle_close(self):
            container.done = True
    
    
    mo = map_()
    
    c = Handler(asynchia.SocketTransport(mo, a), container)
    b.close()
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    mo.close()
    self.assertEqual(container.done, True)


def dnr_close_write(self, map_):
    mo = map_()
    container = Container()
    container.done = False
    
    a, b = asynchia.util.socketpair()
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
            
            self.transport.set_writeable(True)
        
        def handle_close(self):
            container.done = True
    
    
    mo = map_()
    
    c = Handler(asynchia.SocketTransport(mo, a), container)
    b.close()
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    mo.close()
    self.assertEqual(container.done, True)


def dnr_connfailed(self, map_):
    container = Container()
    container.done = False
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
        
        def handle_connect_failed(self, err):
            container.done = True
    
    mo = map_()
    c = Handler(asynchia.SocketTransport(mo), container)
    try:
        c.transport.connect(('wrong.invalid', 81))
    except socket.error:
        container.done = True
        
    
    s = time.time()
    while not container.done and time.time() < s + 10:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEqual(container.done, True)

def dnr_connfailed2(self, map_):
    container = Container()
    container.done = False
    
    class Handler(asynchia.Handler):
        def __init__(self, transport, container=None):
            asynchia.Handler.__init__(self, transport)
            self.container = container
        
        def handle_connect_failed(salf, err):
            if err != errno.ECONNREFUSED:
                self.assertEqual(True, False)
            container.done = True
        
        def handle_connect(salf):
            self.assertEqual(True, False)
    
    mo = map_()
    c = Handler(asynchia.SocketTransport(mo), container)
    
    acceptor = socket.socket()
    # Random port. Only accept local connections.
    acceptor.bind(('127.0.0.1', 0))
    # We know we'll only get one connection.
    acceptor.listen(1)
    
    name = acceptor.getsockname()
    
    acceptor.close()
    
    try:
        c.transport.connect(name)
    except socket.error:
        container.done = True
    
    s = time.time()
    while not container.done and time.time() < s + 30:
        mo.poll(abs(10 - (time.time() - s)))
    self.assertEqual(container.done, True)

class TestCore(unittest.TestCase):    
    def test_error(self):
        container = Container()
        container.done = False
        
        class Serv(asynchia.Server):
            def __init__(
                self, transport, handlercls=asynchia.Handler
                ):
                asynchia.Server.__init__(self, transport, handlercls)
                self.clients = []
            
            def new_connection(self, handler, addr):
                self.clients.append(handler)
        
        
        class Handler(asynchia.Handler):
            def __init__(self, transport, container=None):
                asynchia.Handler.__init__(self, transport)
                self.container = container
            
            def handle_write(self):
                raise ValueError
            
            # Prevent exception from being suppressed.
            def handle_error(self):
                self.container.done = True
        
        mo = asynchia.maps.DefaultSocketMap()
        s = Serv(asynchia.SocketTransport(mo))
        s.transport.bind(('127.0.0.1', 0))
        # We know we'll only get one connection.
        s.transport.listen(1)
        
        c = Handler(asynchia.SocketTransport(mo), container)
        c.transport.connect(s.transport.socket.getsockname())
        
        c.transport.set_writeable(True)
        
        s = time.time()
        while not container.done and time.time() < s + 10:
            mo.poll(abs(10 - (time.time() - s)))
        self.assertEqual(container.done, True)
    
    
    def test_pingpong(self):
        mo = asynchia.maps.DefaultSocketMap()
        a, b = asynchia.util.socketpair()
        ha = asynchia.Handler(asynchia.SocketTransport(mo, a))
        hb = asynchia.Handler(asynchia.SocketTransport(mo, b))
        
        ha.transport.set_writeable(True)
        hb.transport.set_readable(True)
        
        ha.handle_write = lambda: ha.send(b("Foo"))
        hb.handle_read = lambda: self.assertEqual(hb.recv(3), b('Foo'))
    
    if hasattr(socket, 'socketpair'):
        test_pingpong2 = _override_socketpair(test_pingpong)

def _genfun(map_, test):
    def _fun(self):
        return test(self, map_)
    return _fun

maps = \
     (getattr(asynchia.maps, name) for name in
      [
          'SelectSocketMap',
          'PollSocketMap',
          'EPollSocketMap',
          'KQueueSocketMap',
      ]
      if hasattr(asynchia.maps, name)
      )

wsocketpair = [
    dnr_close, dnr_close_read,dnr_close_write, dnr_connfailed,
    dnr_connfailed2, dnr_forthcoming_wakeup
]

tests = [
    dnr_interrupt, t_changeflag(ctx), t_changeflag(std),
    dnr_remove, dnr_remove2
] + wsocketpair

if hasattr(socket, 'socketpair'):
    tests += map(_override_socketpair, wsocketpair)

i = 0

for m in maps:
    for test in tests:
        name = 'test_' + test.__name__[4:] + '_' + m.__name__ + str(i)
        fun = _genfun(m, test)
        fun.__name__ = name
        setattr(TestCore, name, fun)
        i += 1

if __name__ == '__main__':
    unittest.main()



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

import asynchia
from asynchia.util import EMPTY_BYTES

class Depleted(Exception):
    pass


class Input(object):
    """ Base-class for all Inputs. It implements __add__ to return an
    InputQueue consisting of the two operants. """
    def __init__(self, onclose=None):
        self.closed = self.inited = False
        self.onclose = onclose
    
    def tick(self, sock):
        """ Abstract method to be overridden. This should send the data
        contained in the Input over sock. """
        if self.closed:
            return True, 0
        if not self.inited:
            self.init()
    
    def init(self):
        """ Called before the first tick. """
        self.inited = True
    
    def close(self):
        """ Called after the last tick.
        
        IMPORTANT NOTE: This may be called without the Input being initalised,
        so you need to explicitely check whether it is when you want to clean
        up resources allocated in init. """
        if self.onclose is not None:
            self.onclose(self)
        self.closed = True
    
    def __add__(self, other):
        return InputQueue([self, other])


class InputQueue(Input):
    """ An InputQueue calls the tick method an object until it raises
    InputEOF, then it goes on with the next object in the queue. If no
    object is in the queue anymore, InputQueue.tick raises InputEOF (like
    any other Input that doesn't need data anymore). """
    def __init__(self, inputs=None, onclose=None):
        Input.__init__(self, onclose)
        if inputs is None:
            inputs = []
        self.inputs = inputs
    
    def tick(self, sock):
        """ Call tick method of the first object contained in the queue until
        it raises InputEOF. """
        Input.tick(self, sock)
        while True:
            inp = self.inputs[0]
            done, sent = inp.tick(sock)
            if done:
                self.inputs.pop(0)
                if not self.inputs:
                    self.close()
                    return True, sent
            if sent:
                break
        return False, sent
    
    def eof(self):
        """ Called when the last Input in the queue raises InputEOF.
        
        Return True to prevent InputQueue from raising InputEOF."""
    
    def close(self):
        """ Close all inputs contained in the queue. """
        Input.close(self)
        for inp in self.inputs:
            inp.close()
    
    def add(self, other):
        """ Add input to queue. """
        self.inputs.append(other)
    
    def __iadd__(self, other):
        self.add(other)
        return self
    
    def __nonzero__(self):
        return bool(self.inputs)
    
    def __len__(self):
        return sum(len(inp) for inp in self.inputs)


class StringInput(Input):
    """ Input that bufferedly sends a string. """
    def __init__(self, string, onclose=None):
        Input.__init__(self, onclose)
        
        self.buf = string
        self.length = len(self.buf)
    
    def tick(self, sock):
        """ Send as much of the string as possible. """
        Input.tick(self, sock)
        
        sent = sock.send(self.buf)
        self.buf = self.buf[sent:]
        self.length -= sent
        
        if not self.buf:
            self.close()
        
        return not bool(self.buf), sent
    
    def __len__(self):
        return self.length


class FileInput(Input):
    """ Input that buffers at most buffer_size bytes read from the passed fd,
    and sends them in a buffered way. This can be used to "directly" send data
    contained in a file. """
    def __init__(self, fd, length=None, buffer_size=9096, closing=True,
                 onclose=None):
        Input.__init__(self, onclose)
        
        self.cachedlen = length
        
        self.fd = fd
        self.buffer_size = buffer_size
        self.length = length
        self.closing = closing
        
        self.buf = EMPTY_BYTES
        self.eof = False
    
    def tick(self, sock):
        """ Send as much of the file as possible. """
        Input.tick(self, sock)
        if not self.eof and len(self.buf) < self.buffer_size:
            read = self.fd.read(self.buffer_size - len(self.buf))
            if not read:
                self.eof = True
            self.buf += read
        
        sent = sock.send(self.buf)        
        self.buf = self.buf[sent:]
        self.length -= sent
        
        if self.eof and not self.buf:
            self.close()
        
        return self.eof and not self.buf, sent
    
    def close(self):
        """ If FileInput is closing, close the fd. """
        Input.close(self)
        if self.closing:
            self.fd.close()
    
    @classmethod
    def from_filename(cls, filename, mode='rb', *args, **kwargs):
        """ Same as cls(fd, size, *args, **kwargs) while fd and size
        get constructed from the filename. If the mode attribute
        does not equal 'rb', the size cannot be determined
        efficienty. """
        if mode == 'rb':
            size = os.stat(filename).st_size
        else:
            size = None
        fd = open(filename, mode)
        return cls(fd, size, *args, **kwargs)
    
    def __len__(self):
        if self.cachedlen is not None:
            return self.cachedlen
        else:
            pos = self.fd.tell()
            self.fd.seek(0, 2)
            newpos = self.fd.tell()
            self.cachedlen = newpos - pos
            self.fd.seek(pos)
            return self.cachedlen


class AutoFileInput(FileInput):
    """ Same as FileInput, only that it stores samples to guess how big
    the buffer should be. """
    def __init__(self, fd, length=None, buffer_size=9096, closing=True,
                 samples=None, onclose=None):
        FileInput.__init__(self, fd, length, buffer_size, closing, onclose)
        
        if samples is None:
            self.average = asynchia.util.GradualAverage()
        else:
            self.average = asynchia.util.LimitedAverage(samples)
    
    def tick(self, sock):
        """ See FileInput.tick. """
        sent = FileInput.tick(self, sock)
        self.average.add_value(sent)
        self.buffer_size = int(round(self.average.avg))
        return sent


class FactoryInput(Input):
    """ Call factory method to obtain the next input until the factory
    raises the Depleted exception. """
    def __init__(self, factory, onclose=None):
        Input.__init__(self, onclose)
        self.factory = factory
        self.cur_inp = None
    
    def init(self):
        """ Obtain first input from factory method. """
        Input.init(self)
        self.cur_inp = self.factory()
    
    def tick(self, sock):
        """ Send data from the current input. """
        Input.tick(self, sock)
        while True:
            done, sent = self.cur_inp.tick(sock)
            if done:
                try:
                    self.cur_inp = self.factory()
                except Depleted:
                    self.close()
                    return True, sent
            if sent:
                break
        return False, sent
    
    @staticmethod
    def wrap_iterator(itr_next):
        """ Wrap the next method of an iterable in such a way that whenever
        it raises StopIteration, the wrapper raises Depleted. This is
        convenient when using FactoryCollector to obtain the collectors
        from an iterable. """
        def _wrap():
            try:
                return itr_next()
            except StopIteration:
                raise Depleted
        return _wrap


class Collector(object):
    """ This is the base-class for all collectors. Collectors read up to
    nbytes bytes of data from the protocol passed to them. """
    def __init__(self, onclose=None):
        self.inited = self.closed = False
        self.onclose = onclose
    
    def add_data(self, prot, nbytes):
        """ Override to read at most nbytes from prot and store them in the
        collector. """
        if self.closed:
            return True, 0
        
        if not self.inited:
            self.init()
    
    def close(self):
        """ Called after the last data has been added.
        
        IMPORTANT NOTE: This may be called without the Collector being
        initalised, so you need to explicitely check whether it is when you
        want to clean up resources allocated in init. """
        if self.onclose is not None:
            self.onclose(self)
        self.closed = True
    
    def init(self):
        """ Called before the first data is added to the collector. """
        self.inited = True
    
    def __add__(self, other):
        return CollectorQueue([self, other])


class StringCollector(Collector):
    """ Store data received from the socket in a string. """
    def __init__(self, onclose=None):
        Collector.__init__(self, onclose)
        
        self.intvalue = EMPTY_BYTES
    
    def add_data(self, prot, nbytes):
        """ Write at most nbytes bytes from prot to string. """
        Collector.add_data(self, prot, nbytes)
        
        received = prot.recv(nbytes)
        self.intvalue += received
        return False, len(received)
    
    @property
    def value(self):
        return self.intvalue


class FileCollector(Collector):
    """ Write data received from the socket into a fd. """
    def __init__(self, fd=None, closing=True, autoflush=False, onclose=None):
        Collector.__init__(self, onclose)
        
        self.intvalue = fd
        self.closing = closing
        self.autoflush = autoflush
    
    def add_data(self, prot, nbytes):
        """ Write at most nbytes data from prot to fd. """
        Collector.add_data(self, prot, nbytes)
        
        received = prot.recv(nbytes)
        # Let the error propagate lest it is silently ignored.
        self.intvalue.write(received)
        if self.autoflush:
            self.intvalue.flush()
        return False, len(received)
    
    def close(self):
        """ Close the fd if the FileCollector is closing. """
        Collector.close(self)
        if self.closing:
            self.intvalue.close()
    
    @property
    def value(self):
        return self.intvalue


class DelimitedCollector(Collector):
    """ Collect up to size bytes in collector and raise CollectorFull
    afterwards. """
    def __init__(self, collector, size, onclose=None):
        Collector.__init__(self, onclose)
        self.collector = collector
        self.size = size
    
    def add_data(self, prot, nbytes):
        """ Add data until the received data exceeds size. """
        Collector.add_data(self, prot, nbytes)
        
        done, nrecv = self.collector.add_data(prot, min(self.size, nbytes))
        self.size -= nrecv
        if self.size == 0:
            self.close()
        return (self.size == 0), nrecv
    
    def close(self):
        """ Close wrapped collector. """
        Collector.close(self)
        self.collector.close()
    
    def init(self):
        """ Initialise wrapped collector. """
        Collector.init(self)
        self.collector.init()
    
    @property
    def value(self):
        return self.collector.value


class CollectorQueue(Collector):
    """ Write data to the first collector until CollectorFull is raised,
    afterwards repeat with next. When the CollectorQueue gets empty it
    raises CollectorFull. """
    def __init__(self, collectors=None, onclose=None):
        Collector.__init__(self, onclose)
        if collectors is None:
            collectors = []
        self.collectors = collectors
    
    def add_collector(self, coll):
        """ Add coll to queue. """
        self.collectors.append(coll)
    
    def add_data(self, prot, nbytes):
        """ Add data to first collector until it is full. """
        Collector.add_data(self, prot, nbytes)
        while True:
            done, nrecv = self.collectors[0].add_data(prot, nbytes)
            if done:
                self.finish_collector(self.collectors.pop(0))
                if not self.collectors:
                    # Returning 
                    if not self.full():
                        # For the sake of consistency.
                        if not self.closed:
                            self.close()
                        return True, nrecv
            if nrecv:
                break
        return False, nrecv
    
    def finish_collector(self, coll):
        """ Called when a collector raises CollectorFull. """
        pass
    
    def full(self):
        """ Called when the last collector in the queue raises
        CollectorFull.
        
        Return True to prevent CollectorQueue from raising CollectorFull. """
    
    def close(self):
        """ Close collectors contained in the queue. """
        Collector.close(self)
        for collector in self.collectors:
            collector.close()
    
    @property
    def value(self):
        return None


class KeepingCollectorQueue(CollectorQueue):
    """ CollectorQueue that does not discard the collectors contained within
    after they have finished but rather appends to them to the list accessible
    through its member collected. """
    def __init__(self, collectors=None, onclose=None):
        CollectorQueue.__init__(self, collectors, onclose)
        self.collected = []
        
        self.intvalue = []
    
    def finish_collector(self, coll):
        """ Append finished collector to self.collected. """
        self.collected.append(coll)
        self.intvalue.append(coll.value)
        
    @property
    def value(self):
        return self.intvalue


class FactoryCollector(Collector):
    """ Call factory method to obtain the next collector until the factory
    raises the Depleted exception. """
    def __init__(self, factory, onclose=None):
        Collector.__init__(self, onclose)
        self.factory = factory
        self.cur_coll = None
    
    def init(self):
        """ Obtain first collector from factory method. """
        Collector.init(self)
        self.cur_coll = self.factory()
    
    def add_data(self, prot, nbytes):
        """ Add data to the current collector. """
        Collector.add_data(self, prot, nbytes)
        while True:
            done, nrecv = self.cur_coll.add_data(prot, nbytes)
            if done:
                try:
                    self.cur_coll = self.factory()
                # Wise?
                except Depleted:
                    # Redundant if?
                    if not self.closed:
                        self.close()
                    return True, nrecv
            if nrecv:
                break
        return False, nrecv
    
    @staticmethod
    def wrap_iterator(itr_next):
        """ Wrap the next method of an iterable in such a way that whenever
        it raises StopIteration, the wrapper raises Depleted. This is
        convenient when using FactoryCollector to obtain the collectors
        from an iterable. """
        def _wrap():
            try:
                return itr_next()
            except StopIteration:
                raise Depleted
        return _wrap
    
    @property
    def value(self):
        return None


class StructCollector(DelimitedCollector):
    """ Collect and unpack stru. Unpacked value can be found in .value and
    is available upon calling onclose. """
    def __init__(self, stru, onclose=None):
        DelimitedCollector.__init__(
            self, StringCollector(), stru.size, onclose
        )
        
        self.stru = stru
        self.intvalue = None
    
    def close(self):
        """ Unpack the data received and close the collector afterwards. """
        self.intvalue = self.stru.unpack(self.collector.value)
        DelimitedCollector.close(self)
    
    @property
    def value(self):
        return self.intvalue


class SingleStructValueCollector(StructCollector):
    @property
    def value(self):
        return self.intvalue[0]
    
class Handler(asynchia.Handler):
    """ asynchia handler that adds all received data to a collector,
    and reads outgoing data from an Input. """
    def __init__(self, transport, collector=None,
                 buffer_size=9046):
        asynchia.Handler.__init__(self, transport)
        
        self.queue = InputQueue()
        self.collector = collector
        self.buffer_size = buffer_size
        
        if collector is not None:
            self.transport.set_readable(True)
    
    def set_collector(self, collector, noclose=False):
        """ Set the top-level collector to collector. If noclose is False,
        the previous collector's close method is called. """
        if not noclose and self.collector is not None:
            self.collector.close()
        self.collector = collector
        if not self.transport.readable:
            self.transport.set_readable(True)
    
    def send_input(self, inp):
        """ Add inp to the main input queue. """
        self.queue.add(inp)
        if not self.transport.writeable:
            self.transport.set_writeable(True)
    
    def send_str(self, string):
        """ Convenience method for .send_input(StringInput(string)) """
        self.send_input(StringInput(string))
    
    def handle_read(self):
        """ Do the read call. """
        done, nrecv = self.collector.add_data(
            self.transport, self.buffer_size
        )
        if done:
            # We can safely assume it is readable here.
            self.transport.set_readable(False)
    
    def handle_write(self):
        """ Do the write call. """
        done, sent = self.queue.tick(self.transport)
        if done:
            # We can safely assume it is writeable here.
            self.transport.set_writeable(False)
    
    def has_data(self):
        """ Tell whether the Handler has any data to be sent. """
        return bool(self.queue)
    
    def close(self):
        """ Close top-level collector. """
        self.collector.close()
    
    def handle_error(self):
        raise


class MockHandler(object):
    """ This mocks a handler by writing everything that's passed
    to its send method to outbuf, while reading data from inbuf
    upon recv calls. """
    def __init__(self, inbuf=EMPTY_BYTES, outbuf=EMPTY_BYTES):
        self.outbuf = outbuf
        self.inbuf = inbuf
    
    def recv(self, bufsize):
        """ Return up to bufsize bytes from inbuf. Raise ValueError
        when inbuf is empty. """
        if not self.inbuf:
            return EMPTY_BYTES
        i = min(bufsize, len(self.inbuf))
        data = self.inbuf[:i]
        self.inbuf = self.inbuf[i:]
        return data
    
    def send(self, data):
        """ Write data to outbuf. """
        self.outbuf += data
        return len(data)



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Facilities to refer to data that is not yet available.

Example:

    # Usually acquired by a call that results in network I/O.
    # Global variable for demostration purposes.
    a = DataNotifier()
    def bar():
        # Request result of network I/O.
        blub = yield a
        yield Coroutine.return_(blub)
    def foo():
        # Wait for completion of new coroutine which - in turn - waits
        # for I/O. 
        blub = yield Coroutine.call_itr(bar())
        print "yay %s" % blub
    
    c = Coroutine(foo())
    c.call()
    # Network I/O complete.
    a.submit('blub')
"""

import threading

import asynchia
from asynchia.util import b

class PauseContext(object):
    """ Collection of Coroutines which are currently paused but not waiting
    for any data. They are paused to prevent too much time to be spent in
    them, preventing possibly important I/O from being done. """
    def __init__(self):
        self.paused = []
    
    def unpause(self):
        """ Continue all paused coroutines. """
        for coroutine in self.paused:
            coroutine.call()
        self.paused[:] = []
    
    def pause(self, coroutine):
        """ Add coroutine to the list of paused coroutines. """
        self.paused.append(coroutine)


class Coroutine(object):
    """ Create coroutine from given iterator. Yielding None will pause the co-
    routine until continuation by the given PauseContext. Yielding a
    DataNotifier will send the requested value to the coroutine once
    available. Yielding an instance of Coroutine.return_ will end execution
    of the coroutine and send the return value to any coroutines that may be
    waiting for it or calls any callbacks associated with it. """
    class return_:
        """ Yield an instance of this to signal that the coroutine finished
        with the given return value (defaults to None). """
        def __init__(self, obj=None):
            self.obj = obj
    
    def __init__(self, itr, pcontext=None, datanotifier=None):
        self.itr = itr
        if datanotifier is None:
            datanotifier = DataNotifier()
        self.datanotifier = datanotifier
        self.pcontext = pcontext
    
    def send(self, data):
        """ Send requested data to coroutine. """
        try:
            self.handle_result(self.itr.send(data))
        except StopIteration:
            self.datanotifier.submit(None)
    
    def call(self):
        """ Start (or resume) execution of the coroutine. """
        try:
            self.handle_result(self.itr.next())
        except StopIteration:
            self.datanotifier.submit(None)
    
    def handle_result(self, result):
        """ Internal. """
        if result is None:
            if self.pcontext is not None:
                self.pcontext.pause(self)
            else:
                raise ValueError("No PauseContext.")
        elif isinstance(result, Coroutine.return_):
            self.datanotifier.submit(result.obj)
        else:
            result.add_coroutine(self)
    
    @classmethod
    def call_itr(cls, itr):
        """ Create a coroutine from the given iterator, start it
        and return the DataNotifier. """
        coroutine = cls(itr)
        coroutine.call()
        return coroutine.datanotifier


class DataNotifier(object):
    """ Call registered callbacks and send data to registered coroutines
    at submission of data. """
    def __init__(self, socket_map):
        self.dcallbacks = []
        self.rcallbacks = []
        self.coroutines = []
        self.finished = False
        self.data = None
        
        self.event = threading.Event()
        
        self.wakeup, other = asynchia.util.socketpair()
        self.handler = _ThreadedDataHandler(
            asynchia.SocketTransport(socket_map, other),
            self
        )
    
    def add_coroutine(self, coroutine):
        """ Add coroutine that waits for the submission of this data. """
        if self.data is None:
            self.coroutines.append(coroutine)
        else:
            coroutine.send(self.data)
    
    def add_databack(self, callback):
        """ Add databack (function that receives the the data-notifier data
        upon submission as arguments). """
        if self.data is None:
            self.dcallbacks.append(callback)
        else:
            callback(self.data)
    
    def add_callback(self, callback):
        """ Add callback (function that only receives the data upon
        submission as an argument). """
        if self.data is None:
            self.rcallbacks.append(callback)
        else:
            callback(self, self.data)
    
    def poll(self):
        """ Poll whether result has already been submitted. """
        return self.finished
    
    def submit(self, data):
        """ Submit data; send it to any coroutines that may be registered and
        call any data- and callbacks that may be registered. """
        self.data = data
        for callback in self.dcallbacks:
            callback(data)
        for callback in self.rcallbacks:
            callback(self, data)
        for coroutine in self.coroutines:
            coroutine.send(data)
        
        self.coroutines[:] = []
        self.rcallbacks[:] = []
        self.dcallbacks[:] = []
        
        # Wake up threads waiting for the data.
        self.event.set()
        
        self.finished = True
    
    def inject(self, data):
        """ Submit data and ensure their callbacks are called in the main
        thread. """
        self.injected = data
        self.wakeup.send(b('a'))
    
    def wait(self, timeout=None):
        """ Block execution of current thread until the data is available.
        Return requested data. """
        self.event.wait(timeout)
        return self.data
    
    @staticmethod
    def _coroutine(datanotifier, fun, args, kwargs):
        """ Implementation detail. """
        datanotifier.inject(fun(*args, **kwargs))
    
    @classmethod
    def threaded_coroutine(cls, socket_map, fun, *args, **kwargs):
        """ Run fun(*args, **kwargs) in a thread and return a DataNotifier
        notifying upon availability of the return value of the function. """
        datanot = cls(socket_map)
        threading.Thread(
            target=cls._coroutine, args=(datanot, fun, args, kwargs)
        )
        return datanot


class _ThreadedDataHandler(asynchia.Handler):
    """ Implementation detail. """
    def __init__(self, transport, datanotifier):
        asynchia.Handler.__init__(self, transport)
        self.transport.set_readable(True)
        
        self.datanotifier = datanotifier
    
    def handle_read(self):
        """ Implementation detail. """
        self.transport.recv(1)
        self.datanotifier.submit(self.datanotifier.injected)



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Socket-maps are there to decide which sockets have I/O to be done
and call the appropriate handle functions at the Handlers.

If select.poll is available, you will have PollSocketMap available and
also assigned to DefaultSocketMap, which otherwise defaults to
SelectSocketMap.

If select.epoll is available, you will have EPollSocketMap available and
also assigned to DefaultSocketMap, which otherwise defaults to either
PollSocketMap or SelectSocketMap.

If select.kqueue is available, you will have KQueueSocketMap available and
also assigned to DefaultSocketMap, which otherwise defaults to one
of the other three.
"""

import select
import socket
import errno

import asynchia
from asynchia.util import socketpair, b, EMPTY_BYTES, is_closed

class InterruptContextManager(object):
    """ Allow with socketmap.interrupt() """
    def __init__(self, socket_map, changeflags):
        self.socket_map = socket_map
        self.changeflags = changeflags
    
    def __enter__(self):
        self.socket_map.start_interrupt(self.changeflags)
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        self.socket_map.end_interrupt(self.changeflags)


class ControlSocketSocketMap(asynchia.SocketMap):
    """ Socket-map with an internal socket-pair that can be used to
    interrupt it. """
    def __init__(self, notifier):
        asynchia.SocketMap.__init__(self, notifier)
        # IMPORTANT! These have to be blocking!
        self.controlsender, self.controlreceiver = socketpair()
    
    def interrupt(self, changeflags=False):
        """ Return context manager for the interruption of this
        socket-map. """
        return InterruptContextManager(self, changeflags)
    
    def do_interrupt(self):
        """ Call this in the socket-map when you have found out that there is
        data to read on the controlreceiver. """
        # Read the "s" that started the interrupt
        self.controlreceiver.recv(1)
        # Send the "i" that signals the interrupt succeeded.
        self.controlreceiver.send(b('i'))
        # Read the "e" that will end the interrupt.
        self.controlreceiver.recv(1)


class FragileSocketMap(ControlSocketSocketMap):
    """ The socket-map has to be interrupted before internals are
    changed. """
    def start_interrupt(self, changeflags=False):
        """ See SocketMap.start_interrupt. """
        self.controlsender.send(b('s'))
        self.controlsender.recv(1)
    
    def end_interrupt(self, changeflags=False):
        """ See SocketMap.end_interrupt. """
        self.controlsender.send(b('e'))


class RobustSocketMap(ControlSocketSocketMap):
    """ The socket-map has to be stopped and resumed after internals
    are changed. """
    def __init__(self, notifier=None):
        ControlSocketSocketMap.__init__(self, notifier)
        self.needresp = False
    
    def start_interrupt(self, changeflags=False):
        """ See SocketMap.start_interrupt. """
        if not changeflags:
            # Same as FragileSocketMap.start_interrupt.
            self.controlsender.send(b('s'))
            self.controlsender.recv(1)
        else:
            self.needresp = True
    
    def end_interrupt(self, changeflags=False):
        """ See SocketMap.end_interrupt. """
        if not changeflags:
            # Same as FragileSocketMap.end_interrupt.
            self.controlsender.send(b('e'))
        else:
            self.controlsender.send(b('s'))
            self.controlsender.recv(1)
            self.controlsender.send(b('e'))
    
    def close(self):
        """ Signal the interrupt is finished to any thread that may be
        waiting, as the program, waiting for the thread to finish,
        would not be able to exit otherwise. """
        if self.needresp:
            self.controlreceiver.send(b('i'))
            self.needresp = False
        


class RockSolidSocketMap(ControlSocketSocketMap):
    """ The socket-map automatically refreshes its internal while it's
    "sleeping". """
    def start_interrupt(self, changeflags=False):
        """ See SocketMap.start_interrupt. """
        if not changeflags:
            # Same as FragileSocketMap.start_interrupt.
            self.controlsender.send(b('s'))
            self.controlsender.recv(1)
    
    def end_interrupt(self, changeflags=False):
        """ See SocketMap.end_interrupt. """
        if not changeflags:
            # Same as FragileSocketMap.end_interrupt.
            self.controlsender.send(b('e'))


class SelectSocketMap(FragileSocketMap):
    """ Decide which sockets have I/O to do using select.select. """
    def __init__(self, notifier=None):
        FragileSocketMap.__init__(self, notifier)
        self.writers = []
        self.socket_list = []
        
        self.socket_list.append(self.controlreceiver)
    
    def add_transport(self, handler):
        """ See SocketMap.add_transport. """
        if handler in self.socket_list:
            raise ValueError("Handler %r already in socket map!" % handler)
        self.socket_list.append(handler)
        if handler.readable:
            self.add_reader(handler)
        if handler.writeable:
            self.add_writer(handler)
    
    def del_transport(self, handler):
        """ See SocketMap.del_transport. """
        self.socket_list.remove(handler)
        if handler.readable:
            self.del_reader(handler)
        if handler.writeable:
            self.del_writer(handler)
        if handler.awaiting_connect:
            self.del_writer(handler)
        pass
    
    def add_writer(self, handler):
        """ See SocketMap.add_writer. """
        self.writers.append(handler)
    
    def del_writer(self, handler):
        """ See SocketMap.del_writer. """
        self.writers.remove(handler)
    
    def add_reader(self, handler):
        """ See SocketMap.add_reader. """
        pass
    
    def del_reader(self, handler):
        """ See SocketMap.del_reader. """
        pass
    
    def poll(self, timeout):
        """ Poll for I/O. """
        interrupted = False
        
        try:
            read, write, expt = select.select(self.socket_list,
                                              self.writers,
                                              self.socket_list, timeout)
        except IOError, err:
            if err.args[0] == errno.EINTR:
                return
            else:
                raise

        for obj in read:
            if obj != self.controlreceiver:
                # This seems to be the only way to find hangup-events with
                # select.
                if obj.connected and is_closed(obj.socket):
                    self.notifier.close_obj(obj)
                else:
                    self.notifier.read_obj(obj)
            else:
                interrupted = True
        for obj in write:
            self.notifier.write_obj(obj)
        for obj in expt:
            self.notifier.except_obj(obj)
        
        if interrupted:
            self.do_interrupt()
    
    def run(self):
        """ Periodically poll for I/O. """
        while True:
            self.poll(None)
    
    def close(self):
        """ See SocketMap.close """
        for handler in self.socket_list[1:]:
            self.notifier.cleanup_obj(handler)


class PollSocketMap(RobustSocketMap):
    """ Decide which sockets have I/O to do using select.poll. 
    
    Do not refer to this class without explicitely checking for its existance
    first, it may not exist on some platforms (it is known not to on Windows).
    """
    def __init__(self, notifier=None):
        RobustSocketMap.__init__(self, notifier)
        self.socket_list = {}
        self.poller = select.poll()
        
        self.controlfd = self.controlreceiver.fileno()
        self.poller.register(self.controlfd, select.POLLIN | select.POLLPRI)
    
    def add_transport(self, handler):
        """ See SocketMap.add_transport. """
        fileno = handler.fileno()
        if fileno in self.socket_list:
            raise ValueError("Socket with fileno %d already "
                             "in socket map!" % fileno)
        self.socket_list[fileno] = handler
        self.poller.register(fileno, self.create_flags(handler))
    
    def del_transport(self, handler):
        """ See SocketMap.del_transport. """
        fileno = handler.fileno()
        del self.socket_list[fileno]
        self.poller.unregister(fileno)
    
    def poll(self, timeout):
        """ Poll for I/O. """
        interrupted = False
        
        try:
            active = self.poller.poll(timeout)
        except IOError, err:
            if err.args[0] == errno.EINTR:
                return
            else:
                raise
        
        for fileno, flags in active:
            if fileno == self.controlfd:
                interrupted = True
                continue
            obj = self.socket_list[fileno]
            if flags & (select.POLLIN | select.POLLPRI):
                if obj.connected and is_closed(obj.socket):
                    self.notifier.close_obj(obj)
                else:
                    self.notifier.read_obj(obj)
            if flags & select.POLLOUT:
                self.notifier.write_obj(obj)
            if flags & select.POLLPRI:
                self.notifier.except_obj(obj)
            if flags & (select.POLLHUP | select.POLLERR | select.POLLNVAL):
                self.notifier.close_obj(obj)
        if interrupted:
            self.do_interrupt()
    
    def run(self):
        """ Periodically poll for I/O. """
        while True:
            self.poll(None)
    
    def handler_changed(self, handler):
        """ Update flags for handler. """
        # self.poller.register is compatible to 2.5 whilst
        # self.poller.modify is not.
        self.poller.register(handler.fileno(), self.create_flags(handler))
    
    # We just update the flags of the object, doesn't matter what has
    # changed.
    add_writer = del_writer = add_reader = del_reader = handler_changed
    
    @staticmethod
    def create_flags(handler):
        """ Generate appropriate flags for handler. These flags will
        represent the current state of the handler (if it is readable,
        the flags say so too). """
        flags = (
            select.POLLERR | select.POLLHUP | select.POLLNVAL |
            select.POLLIN | select.POLLPRI
        )
        if handler.writeable or handler.awaiting_connect:
            flags |= select.POLLOUT
        return flags
    
    def close(self):
        """ See SocketMap.close """
        RobustSocketMap.close(self)
        for handler in self.socket_list.itervalues():
            self.notifier.cleanup_obj(handler)


class EPollSocketMap(RockSolidSocketMap):
    """ Decide which sockets have I/O to do using select.epoll. 
    
    Do not refer to this class without explicitely checking for its existance
    first, it may not exist on some platforms (it is known not to on Windows
    and BSD).
    """
    def __init__(self, notifier=None):
        RockSolidSocketMap.__init__(self, notifier)
        self.socket_list = {}
        self.poller = select.epoll()
        
        self.controlfd = self.controlreceiver.fileno()
        self.poller.register(self.controlfd, select.EPOLLIN | select.EPOLLPRI)
    
    def add_transport(self, handler):
        """ See SocketMap.add_transport. """
        fileno = handler.fileno()
        if fileno in self.socket_list:
            raise ValueError("Socket with fileno %d already "
                             "in socket map!" % fileno)
        self.socket_list[fileno] = handler
        self.poller.register(fileno, self.create_flags(handler))
    
    def del_transport(self, handler):
        """ See SocketMap.del_transport. """
        fileno = handler.fileno()
        del self.socket_list[fileno]
        self.poller.unregister(fileno)
    
    def poll(self, timeout):
        """ Poll for I/O. """
        # While select.poll is alright with None, select.epoll expects
        # -1 for no timeout,
        if timeout is None:
            timeout = -1
        
        interrupted = False
        
        try:
            active = self.poller.poll(timeout)
        except IOError, err:
            if err.args[0] == errno.EINTR:
                return
            else:
                raise
        
        for fileno, flags in active:
            if fileno == self.controlfd:
                interrupted = True
                continue
            obj = self.socket_list[fileno]
            if flags & (select.EPOLLIN | select.EPOLLPRI):
                if obj.connected and is_closed(obj.socket):
                    self.notifier.close_obj(obj)
                else:
                    self.notifier.read_obj(obj)
            if flags & select.EPOLLOUT:
                self.notifier.write_obj(obj)
            if flags & select.EPOLLPRI:
                self.notifier.except_obj(obj)
            if flags & (select.POLLHUP | select.POLLERR | select.POLLNVAL):
                self.notifier.close_obj(obj)
        if interrupted:
            self.do_interrupt()
    
    def run(self):
        """ Periodically poll for I/O. """
        while True:
            self.poll(None)
    
    def handler_changed(self, handler):
        """ Update flags for handler. """
        self.poller.modify(handler.fileno(), self.create_flags(handler))
    
    # We just update the flags of the object, doesn't matter what has
    # changed.
    add_writer = del_writer = add_reader = del_reader = handler_changed
    
    @staticmethod
    def create_flags(handler):
        """ Generate appropriate flags for handler. These flags will
        represent the current state of the handler (if it is readable,
        the flags say so too). """
        flags = (
            select.EPOLLERR | select.EPOLLHUP | select.POLLNVAL |
            select.EPOLLIN | select.EPOLLPRI
        )
        if handler.writeable or handler.awaiting_connect:
            flags |= select.EPOLLOUT
        return flags
    
    def close(self):
        """ See SocketMap.close """
        for handler in self.socket_list.itervalues():
            self.notifier.cleanup_obj(handler)


# It is possible to only get hangup events by applying the hack presented
# at http://paste.pocoo.org/show/245033/

# The concept of exceptional socket state does not exist on BSD, to quote
# the manpage of select on FreeBSD:
#     The only exceptional condition detectable is out-of-band data
#     received on a socket.
# Thus we need not call notifier.except_obj in this socket-map.
class KQueueSocketMap(RockSolidSocketMap):
    """ Decide which sockets have I/O to do using select.kqueue. 
    
    Do not refer to this class without explicitely checking for its existance
    first, it may not exist on some platforms (it only exists on BSD).
    """
    def __init__(self, nevents=100, notifier=None):
        RockSolidSocketMap.__init__(self, notifier)
        self.socket_list = {}
        self.queue = select.kqueue()
        
        self.nevents = nevents
        
        self.controlfd = self.controlreceiver.fileno()
        self.queue.control(
            [select.kevent(self.controlfd,
                           select.KQ_FILTER_READ,
                           select.KQ_EV_ADD)], 0
        )
    
    def add_transport(self, handler):
        """ See SocketMap.add_transport. """
        if handler in self.socket_list:
            raise ValueError("Handler %r already in socket map!" % handler)
        self.socket_list[handler.fileno()] = handler
        
        self.queue.control(
            [select.kevent(handler, select.KQ_FILTER_READ, select.KQ_EV_ADD)],
            0
        )
        
        if handler.readable:
            self.add_reader(handler)
        if handler.writeable:
            self.add_writer(handler)
    
    def del_transport(self, handler):
        """ See SocketMap.del_transport. """
        self.socket_list.pop(handler.fileno())
        
        self.queue.control(
            [select.kevent(
                handler, select.KQ_FILTER_READ, select.KQ_EV_DELETE)
             ],
            0
        )
        
        if handler.readable:
            self.del_reader(handler)
        if handler.writeable or handler.awaiting_connect:
            self.del_writer(handler)
    
    def add_reader(self, handler):
        """ See SocketMap.add_reader. """
        pass
    
    def del_reader(self, handler):
        """ See SocketMap.del_reader. """
        pass
    
    def add_writer(self, handler):
        """ See SocketMap.add_writer. """
        self.queue.control(
            [select.kevent(handler, select.KQ_FILTER_WRITE, select.KQ_EV_ADD)],
            0
        )
    
    def del_writer(self, handler):
        """ See SocketMap.del_writer. """
        self.queue.control(
            [select.kevent(
                handler, select.KQ_FILTER_WRITE, select.KQ_EV_DELETE)
             ],
            0
        )
    
    def poll(self, timeout=None):
        interrupted = False
        
        try:
            res = self.queue.control(None, self.nevents, timeout)
        except IOError, err:
            if err.args[0] == errno.EINTR:
                return
            else:
                raise
        
        for event in res:
            if event.ident == self.controlfd:
                interrupted = True
                continue
            handler = self.socket_list[event.ident]
            if event.filter == select.KQ_FILTER_READ:
                self.notifier.read_obj(handler)
            if event.filter == select.KQ_FILTER_WRITE:
                self.notifier.write_obj(handler)
            if event.flags & select.KQ_EV_EOF:
                self.notifier.close_obj(handler)
        
        if interrupted:
            self.do_interrupt()
    
    def close(self):
        self.queue.close()
        for handler in self.socket_list.itervalues():
            self.notifier.cleanup_obj(handler)


DefaultSocketMap = SelectSocketMap

if hasattr(select, 'poll'):
    DefaultSocketMap = PollSocketMap
else:
    del PollSocketMap

if hasattr(select, 'epoll'):
    DefaultSocketMap = EPollSocketMap
else:
    del EPollSocketMap

if hasattr(select, 'kqueue'):
    DefaultSocketMap = KQueueSocketMap
else:
    del KQueueSocketMap



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import string
import struct
import tempfile

import asynchia.ee
from asynchia.util import b

import unittest

def get_named_tempfile(delete):
    try:
        return tempfile.NamedTemporaryFile(delete=delete)
    except TypeError:
        f = tempfile.NamedTemporaryFile()
        if not delete and os.name != "nt":
            def close():
                f.close_called = True
                f.file.close()
                f.close = close
        return f


def del_strcoll(size):
    return asynchia.ee.DelimitedCollector(
        asynchia.ee.StringCollector(), size
    )

def until_done(fun):
    while True:
        d, s = fun()
        if d:
            break

class TestEE(unittest.TestCase):
    def test_inputqueue(self):
        m = asynchia.ee.MockHandler()
        a = asynchia.ee.StringInput(b('a') * 5)
        c = asynchia.ee.StringInput(b('b') * 5)
        d = asynchia.ee.StringInput(b('c') * 5)
        q = asynchia.ee.InputQueue([a, c, d])
        
        until_done(lambda: q.tick(m))
        self.assertEqual(m.outbuf, b('a') * 5 + b('b') * 5 + b('c') * 5)
    
    
    def test_stringinput(self):
        m = asynchia.ee.MockHandler()
        i = asynchia.ee.StringInput(b(string.ascii_letters))
        i.tick(m)
        self.assertEqual(m.outbuf, b(string.ascii_letters))
    
    
    def test_fileinput(self):
        data = open(__file__, 'rb').read()
        m = asynchia.ee.MockHandler()
        i = asynchia.ee.FileInput.from_filename(__file__)
        self.assertEqual(len(i), len(data))
        until_done(lambda: i.tick(m))
        i.close()
        self.assertEqual(m.outbuf, data)
    
    
    def test_fileinput_closing(self):
        i = asynchia.ee.FileInput.from_filename(__file__)
        i.close()
        # I/O operation on closed file.
        self.assertRaises(ValueError, i.fd.read, 6)
    
    
    def test_notclosing(self):
        i = asynchia.ee.FileInput.from_filename(__file__, closing=False)
        i.close()
        # Verify file is not closed.
        self.assertEqual(i.fd.read(0), b(''))
    
    
    def test_filecollector_closing(self):
        c = asynchia.ee.FileCollector(
            get_named_tempfile(delete=False)
        )
        m = asynchia.ee.MockHandler(inbuf=b(string.ascii_letters))
        c.add_data(m, 10)
        c.close()
        # I/O operation on closed file.
        self.assertRaises(ValueError, c.value.read, 6)
    
    
    def test_filecollector_notclosing(self):
        c = asynchia.ee.FileCollector(
            get_named_tempfile(delete=False),
            False
        )
        m = asynchia.ee.MockHandler(inbuf=b(string.ascii_letters))
        c.add_data(m, 10)
        c.close()
        c.value.seek(0)
        r = b('')
        while len(r) < 10:
            r += c.value.read(10 - len(r))
        self.assertEqual(r, b(string.ascii_letters[:10]))
    
    
    def test_delimited(self):
        c = asynchia.ee.DelimitedCollector(
            asynchia.ee.StringCollector(), 5
        )
        m = asynchia.ee.MockHandler(inbuf=b(string.ascii_letters))
        n = c.add_data(m, 10)
        self.assertEqual(n[1], 5)
        # As we are using a MockHandler, we can be sure the collector
        # collected all 5 bytes it was supposed to.
        self.assertEqual(c.add_data(m, 10)[0], True)
        # The collector
        self.assertEqual(c.collector.value, b(string.ascii_letters[:5]))
        self.assertEqual(m.inbuf, b(string.ascii_letters[5:]))
        
    
    def test_collectorqueue(self):
        a = asynchia.ee.DelimitedCollector(
            asynchia.ee.StringCollector(), 5
        )
        c = asynchia.ee.DelimitedCollector(
            asynchia.ee.StringCollector(), 4
        )
        d = asynchia.ee.DelimitedCollector(
            asynchia.ee.StringCollector(), 3
        )
        
        q = asynchia.ee.CollectorQueue([a, c, d])
        
        m = asynchia.ee.MockHandler(inbuf=b('a') * 5 + b('b') * 4 + b('c') * 3)
        until_done(lambda: q.add_data(m, 5))
        self.assertEqual(a.collector.value, b('a') * 5)
        self.assertEqual(c.collector.value, b('b') * 4)
        self.assertEqual(d.collector.value, b('c') * 3)
    
    
    def test_factorycollector(self):
        def make_eq(i):
            def eq(c):
                return self.assertEqual(c.value, b(5 * string.ascii_letters[i]))
            return eq
        itr = (asynchia.ee.DelimitedCollector(
            asynchia.ee.StringCollector(make_eq(i)), 5) for i in xrange(3))
        c = asynchia.ee.FactoryCollector(
            asynchia.ee.FactoryCollector.wrap_iterator(itr.next)
            )
        m = asynchia.ee.MockHandler(
            inbuf=b('a') * 5 + b('b') * 5 + b('c') * 5 + b('d'))
        until_done(lambda: c.add_data(m, 5))
        self.assertEqual(c.add_data(m, 1)[0], True)
        
    
    def test_factoryinput(self):
        itr = (asynchia.ee.StringInput(b(5 * string.ascii_letters[i]))
               for i in xrange(3))
        c = asynchia.ee.FactoryInput(
            asynchia.ee.FactoryInput.wrap_iterator(itr.next)
            )
        m = asynchia.ee.MockHandler()
        until_done(lambda: c.tick(m))
        self.assertEqual(m.outbuf, b('a') * 5 + b('b') * 5 + b('c') * 5)
        self.assertEqual(c.tick(m)[0], True)
    
    
    def test_close(self):
        c = asynchia.ee.DelimitedCollector(asynchia.ee.StringCollector(), 5)
        m = asynchia.ee.MockHandler(b('abcde'))
        c.add_data(m, 10)
        self.assertEqual(c.closed, True)
    
    
    def test_inputadd(self):
        m = asynchia.ee.MockHandler()
        q = asynchia.ee.StringInput(b('a')) + asynchia.ee.StringInput(b('b'))
        id1 = id(q)
        q += asynchia.ee.StringInput(b('c'))
        id2 = id(q)
        self.assertEqual(id1, id2)
        until_done(lambda: q.tick(m))
        self.assertEqual(m.outbuf, b('abc'))
    
    
    def test_closeinqueue(self):
        q = asynchia.ee.InputQueue()
        a = asynchia.ee.Input()
        q.add(a)
        q.close()
        self.assertEqual(a.closed, True)
    
    
    def test_lenpredict(self):
        strings = [b('a') * i for i in xrange(1, 20)]
        for string in strings:
            fd = get_named_tempfile(delete=True)
            try:
                fd.write(string)
                fd.flush()
                fd.seek(0)
                c = asynchia.ee.FileInput(fd)
                self.assertEqual(len(c), len(string))
            finally:
                fd.close()
    
    
    def test_fromfilename(self):
        strings = [
            b('a' + '\n') * i + b('b' + '\r\n') * j
            for i in xrange(1, 20)
            for j in xrange(1, 20)
        ]
        for string in strings:
            fd = get_named_tempfile(delete=True)
            try:
                fd.write(string)
                fd.flush()
                fd.seek(0)
                c = asynchia.ee.FileInput.from_filename(fd.name, 'r')
                self.assertEqual(len(c), len(string))
            finally:
                fd.close()
    
    
    def test_collectoradd(self):
        a = del_strcoll(5)
        c = del_strcoll(6)
        
        q = a + c
        m = asynchia.ee.MockHandler(b('a') * 5 + b('b') * 6)
        until_done(lambda: q.add_data(m, 2))
        self.assertEqual(a.collector.value, b('a') * 5)
        self.assertEqual(c.collector.value, b('b') * 6)
    
    
    def test_autoflush(self):
        fd = get_named_tempfile(delete=True)
        fc = asynchia.ee.FileCollector(fd, autoflush=True)
        
        i = 0
        m = asynchia.ee.MockHandler(b('a') * 20000000)
        
        while m.inbuf:
            d, n = fc.add_data(m, 8000000)
            i += n
            
            self.assertEqual(os.stat(fd.name).st_size, i)
    
    
    def test_strucollector(self):
        s = struct.Struct('!dh')
        c = asynchia.ee.StructCollector(s)
        m = asynchia.ee.MockHandler(s.pack(14, 25))
        until_done(lambda: c.add_data(m, 2))
        self.assertEqual(c.value, (14, 25))

if __name__ == '__main__':
    unittest.main()



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import random

import asynchia.util

b = asynchia.util.b

class TestUtil(unittest.TestCase):
    def test_socketpair(self):
        data = b('a')
        a, c = asynchia.util.socketpair()
        a.send(data)
        # One byte must at least be received.
        self.assertEqual(c.recv(len(data)), data)
    
    
    def test_idpool(self):
        pool = asynchia.util.IDPool()
        [self.assertEqual(pool.get(), n) for n in xrange(3)]
        pool.release(1)
        self.assertEqual(pool.get(), 1)
        pool.reset()
        [self.assertEqual(pool.get(), n) for n in xrange(3)]
        pool.release(1)
        self.assertEqual(pool.get(), 1)
        [pool.release(n) for n in xrange(3)]
        self.assertEqual(pool.free_ids, [])
    
    
    def test_ipv4(self):
        self.assertEqual(
            asynchia.util.parse_ipv4('127.0.0.1:12345'), ('127.0.0.1', 12345)
        )
        self.assertEqual(
            asynchia.util.parse_ipv4('127.0.0.1'), ('127.0.0.1', -1)
        )
    
    
    def test_ipv6(self):
        self.assertEqual(
            asynchia.util.parse_ipv6(
                '[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:443'
                ),
            ('2001:0db8:85a3:08d3:1319:8a2e:0370:7344', 443)
        )
        self.assertEqual(
            asynchia.util.parse_ipv6(
                '2001:0db8:85a3:08d3:1319:8a2e:0370:7344'
                ),
            ('2001:0db8:85a3:08d3:1319:8a2e:0370:7344', -1)
        )
    
    
    def test_ip(self):
        self.assertEqual(
            asynchia.util.parse_ip('127.0.0.1:12345'), ('127.0.0.1', 12345)
        )
        self.assertEqual(
            asynchia.util.parse_ip(
                '[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:443'
                ),
            ('2001:0db8:85a3:08d3:1319:8a2e:0370:7344', 443)
        )
    
    def test_gradualaverage(self):
        avg = asynchia.util.GradualAverage()
        avg2 = asynchia.util.GradualAverage()
        
        s = [random.randint(0, 2000) for _ in xrange(random.randint(1, 20))]
        
        avg.add_values(s)
        for x in s:
            avg2.add_value(x)
        
        self.assertEqual(avg.avg, avg2.avg)
        self.assertEqual(avg.avg, sum(s) / float(len(s)))

    def test_limitedaverage(self):
        avg = asynchia.util.LimitedAverage(10)
        avg2 = asynchia.util.LimitedAverage(10)
        
        s = [random.randint(0, 2000) for _ in xrange(random.randint(12, 20))]
        
        avg.add_values(s)
        for x in s:
            avg2.add_value(x)
        
        self.assertEqual(avg.avg, avg2.avg)
        self.assertEqual(avg.avg, sum(s[-10:]) / float(len(s[-10:])))


if __name__ == '__main__':
    unittest.main()



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2010 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" A domain-specific language to conveniently define the structure of packets
in binary protocols.

An elementary expression is an object implementing the Expr interface (which
is preferably an interface of a subclass of Expr and thus consistenly
integrates with other Exprs). It needs to implement the __call__(state)
and the produce(value) methods where __call__ returns the appropriate
collector to collect the data described by the expression and produce
returns the input that encodes the value passed to it according to the
expression.

By default the following expressions are defined by the module::

    - BinaryExpr (BE)
    - SingleBinaryExpr (SBE)
    - StringExpr (SE)
    - FixedLenExpr (FLE)
    - FileExpression (FE)

Two methods are provided to lookback data earlier collected in order to
construct the collector. binarylookback(ind, item=0) (also exported as bl)
gets the item specified from the binary data parsed from the expression at
the index ind (zero-index) of the ExprAdd the respective expression is
contained within. It is also possible to use positions relative to the
current one (-1 is the element before the one the lookback is passed to).
singlebinarylookback (also exported as sbl) does likewise for
SingleBinaryExpr (and is arguably more useful, see the Example section).
The only Expr in this module that takes a lookback as a parameter is
FixedLenExpr which limits whichever expression is contained in it
to the number of bytes aquired by the lookback function.

Attributes can be named by passing the name in square brackets. It is
also possible to refer to expressions by the name given to them in
lookbacks. ::

    e = s.L()['size'] + s.B()['blub'] + LFLSE('size')['string']

The tuple of values collected can be converted into a dictionary that
maps the name of the expression to the respective value by calling the
tonamed method of the ExprAdd. ::

    a = e()
    [Collecting of data takes place]
    tup = a.values
    dic = e.tonamed(tup)

Example
=======
Packets are described by expressions which can be converted to collectors
whenever a packet of that type should be parsed. Complex expressions can
be created by adding expressions, resulting in ExprAdds (but that is semi-
internal API which you do not necessarily need to care about, unless you
try to write your own types of expressions).

Let us henceforth consider the following simple example::
    
    from asynchia.dsl import s, SBLFLSE
    e = s.L() + s.B() + LFLSE(0)

This might appear utterly complicated at first glance, but it is not. The
first statement imports s (which is a container for binary numeric types)
and LFLSE which expands to lookback fixed-length string-expression.

The expression (which is the second statement) describes a packet which
contains three parts. The first part is an unsigned long (which is named by its
name in the struct module) in network byte-order (all binary types contained
in `s` assume network byte-order (so s.L expands to the struct format "!L");
the second part is an unsigned byte; the interesting thing in the expression
is the third part which describes a fixed-length string with the length equal
to the first element in the expression (which is referred to by its index 0):
the unsigned long.

The expression can now be used to parse and to produce packets according to the
given format. An asynchia.ee collector can be created by calling the
expression ::

    collector = e()

You can now pass the protocol to the add_data method of the collector and it
will collect the data as specified. Any other data (which appears after the
whole packet has been read) is left in the protocol.

To create a packet you need to call the expression's produce method and pass
a tuple of the data you want to construct a package of. This operation
returns an asynchia.ee.Input that can directly be passed to be sent by
an asynchia.Handler. ::

    data = e.produce((5, 2, 'ABCDE'))

In this case it is important that the first number equals the length of the
string, as the system does not derive it from the length of the string
(though you are advised to do so in your client code).
"""

import struct

import asynchia.ee
from asynchia.util import b

class Container(object):
    """ Container to hold members, which are also exposed through getitem.
    Used as a container for the preconstruced SingleBinaryExpressions. """
    def __getitem__(self, item):
        return getattr(self, item)


class State(object):
    """ State of the collection done by ExprCollectorQueue. Used to enable
    lookbacks. """
    def __init__(self):
        self.tbl = {}
        self.ind = 0
        self.nametbl = {}


class Expr(object):
    """ Baseclass for all expressions. """
    def __init__(self):
        self.name = None
    
    def __add__(self, other):
        return ExprAdd(self, other)
    
    def __getitem__(self, other):
        self.name = other
        return self


class ExprCollectorQueue(asynchia.ee.Collector):
    """ Queue to collect the data specified by the list of expressions
    passed to it. """
    def __init__(self, exprs, onclose=None):
        asynchia.ee.Collector.__init__(self, onclose)
        
        self.exprs = exprs
        self.done = []
        
        self.state = State()
        self.state.tbl = self.done
        
        self.coll = None
    
    def __add__(self, other):
        self.exprs.append(other)
        return self
    
    def add_data(self, prot, nbytes):
        asynchia.ee.Collector.add_data(self, prot, nbytes)
        if self.coll is None:
            self.expr = self.exprs.pop(0)
            self.coll = self.expr(self.state)
        while True:
            done, nrecv = self.coll.add_data(prot, nbytes)
            if done:
                self.done.append(self.coll)
                if self.expr.name is not None:
                    self.state.nametbl[self.expr.name] = self.coll
                self.state.ind += 1
                if not self.exprs:
                    if not self.closed:
                        self.close()
                    return True, nrecv
                else:
                    self.coll = None
            if nrecv:
                break
        return False, nrecv
    
    def __iter__(self):
        return iter(self.done)
    
    @property
    def value(self):
        return (elem.value for elem in self.done)


class ExprAdd(Expr):
    def __init__(self, one, other):
        Expr.__init__(self)
        self.exprs = [one, other]
    
    def __call__(self, state=None, onclose=None):
        # We need to pass a copy so ExprCollectorQueue does not pop
        # from this list. Consider creating the copy in
        # ExprCollectorQueue.__init__.
        return ExprCollectorQueue(self.exprs[:], onclose)
    
    def __add__(self, other):
        self.exprs.append(other)
        return self
    
    def produce(self, value):
        result = asynchia.ee.StringInput(b(""))
        for expr, elem in zip(self.exprs, value):
            result += expr.produce(elem)
        return result
    
    def tonamed(self, tup):
        d = {}
        for expr, elem in zip(self.exprs, tup):
            if expr.name is not None:
                d[expr.name] = elem
        return d


class BinaryExpr(Expr):
    """ Represents data that can be expressed by a Struct. """
    def __init__(self, pattern):
        Expr.__init__(self)
        self.pattern = pattern
    
    def __call__(self, state, onclose=None):
        return asynchia.ee.StructCollector(
            struct.Struct(self.pattern),
            onclose
        )
    
    def produce(self, value):
        return asynchia.ee.StringInput(
            struct.pack(self.pattern, *value)
        )


class SingleBinaryExpr(Expr):
    """ Represents data that can be expressed by a Struct with a single
    member. """
    def __init__(self, pattern):
        Expr.__init__(self)
        self.pattern = pattern
    
    def __call__(self, state, onclose=None):
        return asynchia.ee.SingleStructValueCollector(
            struct.Struct(self.pattern),
            onclose
        )
    
    def produce(self, value):
        return asynchia.ee.StringInput(
            struct.pack(self.pattern, value)
        )


class FileExpr(Expr):
    """ Data that should be collected in an fd. Should be combined with a
    FixedLengthExpression, lest it greedily collects all data from the
    moment it is created until no data is available. """
    def __init__(self, fd, closing=False, autoflush=True):
        Expr.__init__(self)
        self.fd = fd
        self.closing = closing
        self.autoflush = autoflush
    
    def __call__(self, state, onclose=None):
        return asynchia.ee.FileCollector(
            self.fd, self.closing, self.autoflush, onclose
        )
    
    @staticmethod
    def produce(value):
        return asynchia.ee.FileInput(value, closing=False)


class StringExpr(Expr):
    """ Data that should be collected in a string. Should be combined with a
    FixedLengthExpression, lest it greedily collects all data from the
    moment it is created until no data is available. """
    def __call__(self, state, onclose=None):
        return asynchia.ee.StringCollector(onclose)
    
    @staticmethod
    def produce(value):
        return asynchia.ee.StringInput(value)


class FixedLenExpr(Expr):
    """ Restrict the length of the expression contained to the value retuned
    by calling glen upon construction of the collector for said expression.
    """
    def __init__(self, glen, expr):
        Expr.__init__(self)
        self.glen = glen
        self.expr = expr
    
    def __call__(self, state, onclose=None):
        return asynchia.ee.DelimitedCollector(
            self.expr(state), self.glen(state), onclose
        )
    
    def produce(self, value):
        return self.expr.produce(value)


def lookback(ind, fun=(lambda x: x.value)):
    """ Can be passed to expressions that expect a lookback.
    ind can either be a string, thus referring to the result of the expression
    with the specified name or an int referring to the result of the
    expression with the specified index.
    
    Fun can be altered to change the way the data is extracted from the 
    collector. Defaults to returning .value."""
    if isinstance(ind, (long, int)):
        def _fun(state):
            return fun(state.tbl[ind])
    else:
        def _fun(state):
            return fun(state.nametbl[ind])
    return _fun


def binarylookback(ind, item=0):
    """ Convenience function for lookback(ind, lambda x: x.value[item]). """
    return lookback(ind, lambda x: x.value[item])


def const(value):
    """ Can be passed to expressions that expect a lookback and always
    returns the value passed to it. """
    def _fun(state):
        return value
    return _fun


#: Binary lookback
bl = binarylookback
lb = lookback
#: Fixed-length expression
FLE = FixedLenExpr
#: String expression
SE = StringExpr
#: Binary expression
BE = BinaryExpr
SBE = SingleBinaryExpr
FE = FileExpr


def FLSE(glen):
    return FixedLenExpr(glen, StringExpr())

#: Lookback fixed-length string-expression
def LFLSE(ind, fun=(lambda x: x.value)):
    return FixedLenExpr(lookback(ind, fun), StringExpr())


FRMT_CHARS = ('x', 'c', 'b', 'B', '?', 'h', 'H', 'i', 'I', 'l',
              'L', 'q', 'Q', 'f', 'd', 's', 'p', 'P')
s = Container()

def _single_binary(symbol):
    def _fun():
        return SingleBinaryExpr(symbol)
    return _fun

for symbol in FRMT_CHARS:
    setattr(s, symbol, _single_binary("!" + symbol))



# -*- coding: us-ascii -*-

# asynchia.qtmap - SocketMap for Qt
# Copyright (C) 2008 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" This module is - unlike the rest of asynchia - licensed under the terms
of the GNU GPL! """

import asynchia
import socket
import errno

from PyQt4 import QtCore, QtGui


class SocketNotifier(QtCore.QSocketNotifier):
    """ Notificate about socket I/O using Qt facilities """
    def __init__(self, watched, notifier, type_ ):
        QtCore.QSocketNotifier.__init__(self, watched.transport.fileno(), type_)
        self.watched = watched
        self.notifier = notifier
        self.fun = None
        if type_ == QtCore.QSocketNotifier.Read:
            self.fun = self.read
        elif type_ == QtCore.QSocketNotifier.Write:
            self.fun = self.write
        elif type_ == QtCore.QSocketNotifier.Exception:
            self.fun = self.except_
        QtCore.QObject.connect(self, QtCore.SIGNAL("activated(int)"), self.fun)
    
    def disable(self):
        """ Temporarily disable notification on I/O. """
        self.setEnabled(False)
    
    def enable(self):
        """ Re-enable notification on I/O. """
        self.setEnabled(True)
    
    def shutdown(self):
        """ Permanentely disable notification on I/O. """
        QtCore.QObject.disconnect(
            self, QtCore.SIGNAL("activated(int)"), self.fun
        )
        self.setEnabled(False)
        self.fun = self.watched = None
        self.deleteLater()
    
    def read(self, _sock=None):
        """ Read I/O to do. """
        self.notifier.read_obj(self.watched)
    
    def write(self, _sock=None):
        """ Write I/O to do. """
        try:
            self.watched.socket.getpeername()
        except socket.error, err:
            if err.args[0] == errno.ENOTCONN:
                return
            else:
                raise
        self.notifier.write_obj(self.watched)
    
    def except_(self, _sock=None):
        """ Exception I/O to do. """
        self.notifier.except_obj(self.watched)


class QSocketMap(asynchia.SocketMap):
    """ Decide which sockets have I/O to do using Qt facilities. """
    def __init__(self, notifier=None):
        asynchia.SocketMap.__init__(self, notifier)
        self.handler_map = {}
    
    def add_transport(self, handler):
        """ See asynchia.SocketMap.add_transport """
        read = SocketNotifier(
            handler, self.notifier, QtCore.QSocketNotifier.Read
        )
        write = SocketNotifier(
            handler, self.notifier, QtCore.QSocketNotifier.Write
        )
        exception = SocketNotifier(
            handler, self.notifier, QtCore.QSocketNotifier.Exception
        )
        
        if handler.readable:
            read.enable()
        else:
            read.disable()
        
        if handler.writeable:
            write.enable()
        else:
            write.disable()
        exception.enable()
        
        self.handler_map[handler] = {
            'read': read,
            'write': write,
            'exception': exception
        }
    
    def del_transport(self, handler):
        """ See asynchia.SocketMap.del_transport """
        for notifier in self.handler_map[handler].iteritems():
            notifier.shutdown()
        del self.handler_map[handler]
    
    def add_reader(self, handler):
        """ See asynchia.SocketMap.add_reader """
        self.handler_map[handler]['read'].enable()
    
    def del_reader(self, handler):
        """ See asynchia.SocketMap.del_reader """
        self.handler_map[handler]['read'].disable()
    
    def add_writer(self, handler):
        """ See asynchia.SocketMap.add_writer """
        self.handler_map[handler]['write'].enable()
    
    def del_writer(self, handler):
        """ See asynchia.SocketMap.del_writer """
        self.handler_map[handler]['write'].disable()
    
    def close(self):
        """ See asynchia.SocketMap.close """
        for handler in self.handler_map:
            self.notifier.cleanup_obj(handler)
        self.handler_map.clear()



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" asynchia is a minimalist asynchronous networking library. """

import os
import errno
import socket

import traceback

from asynchia.util import EMPTY_BYTES
from asynchia.const import trylater, connection_lost

__version__ = '0.1.0a'

def _unawait_conn(obj):
    """ Helper function for Notifier. """
    obj.stop_awaiting_connect()
    obj.connected = True
    # Errno of the asynchronous connect function.
    err = obj.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
    if err:
        try:
            obj.connected = False
            obj.handle_connect_failed(err)
        except Exception:
            obj.handle_error()
    else:
        try:
            obj.handle_connect()
        except Exception:
            obj.handle_error()


class SocketMap(object):
    """ Decide which sockets have I/O to be done and tell the notifier
    to call the appropriate methods of their Handle objects. """
    def __init__(self, notifier=None):
        if notifier is None:
            notifier = Notifier()
        self.notifier = notifier
    
    def add_transport(self, obj):
        """ Add handler to the socket-map. This gives the SocketMap
        the responsibility to call its handle_read, handle_write,
        handle_close and handle_connect upon new I/O events. """
        raise NotImplementedError
    
    def del_transport(self, obj):
        """ Remove handler from the socket-map. It will no longer have its
        handle_read, handle_write, handle_close and handle_connect methods
        called upon new I/O events. """
        raise NotImplementedError
    
    def add_reader(self, obj):
        """ Add handler as a reader.
        This indicates he wishes to read data. """
        raise NotImplementedError
    
    def del_reader(self, obj):
        """ Delete handler as a reader.
        This indicates he no longer wants to read data until added again """
        raise NotImplementedError
    
    def add_writer(self, obj):
        """ Add handler as a writer.
        This indicates he wishes to send data. """
        raise NotImplementedError
    
    def del_writer(self, obj):
        """ Delete handler as a writer.
        This indicates he no longer wants to send data until added again """
        raise NotImplementedError
    
    def start_interrupt(self, changeflags=False):
        """ Stop the socket-map until end_interrupt is called. """
        raise NotImplementedError
    
    def end_interrupt(self, changeflags=False):
        """ Resume the socket-map. """
        raise NotImplementedError
    
    def interrupt(self, changeflags=False):
        """ Return a context-manager that automatically resumes the
        socket-map when the block is left. """
        raise NotImplementedError
    
    def close(self):
        """ Call the handle_cleanup methods of all handlers contained
        in the socket-map, indicating that they are void. """
        raise NotImplementedError


class Notifier(object):
    """ Call handle functions of the object with error handling. """
    @staticmethod
    def read_obj(obj):
        """ Call handle_read of the object. If any error occurs within it,
        call handle_error of the object. If it is the first read event call
        the handle_connect method. """
        if obj.awaiting_connect:
            _unawait_conn(obj)
        
        if not obj.readable:
            # This shouldn't be happening!
            return
        try:
            obj.handle_read()
        except Exception:
            obj.handle_error()
    
    @staticmethod
    def write_obj(obj):
        """ Call handle_write of the object. If any error occurs within it,
        call handle_error of the object. If it is the first write event call
        the handle_connect method. """
        if obj.awaiting_connect:
            _unawait_conn(obj)
        
        if not obj.writeable:
            # This should only be happening if the object was just connected.
            return
        try:
            obj.handle_write()
        except Exception:
            obj.handle_error()
    
    @staticmethod
    def except_obj(obj):
        """ Call handle_except of the object. If any error occurs within it,
        call handle_error of the object.  """
        if obj.awaiting_connect:
            _unawait_conn(obj)
        
        try:
            obj.handle_except(
                obj.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
            )
        except Exception:
            obj.handle_error()
    
    @staticmethod
    def close_obj(obj):
        """ Call handle_close of the object. If any error occurs within it,
        call handle_error of the object.  """
        if obj.awaiting_connect:
            _unawait_conn(obj)
        try:
            obj.handle_close()
        except Exception:
            obj.handle_error()
        
        try:
            obj.handle_cleanup()
        except Exception:
            obj.handle_error()
    
    @staticmethod
    def cleanup_obj(obj):
        """ Call handle_cleanup of the object. If any error occurs within it,
        call handle_error of the object. """
        try:
            obj.handle_cleanup()
        except Exception:
            obj.handle_error()


class Transport(object):
    """ All I/O is done over Transports. This enables Handlers to be written
    in a generic way and used with different transports. """
    def __init__(self, handler=None):
        self.handler = None
        
        self.closed = False
    
    def recv(self, nbytes, *args):
        """ Override. """
        raise NotImplementedError
    
    def send(self, string, *args):
        """ Override. """
        raise NotImplementedError
    
    def handle_read(self):
        """ Handle read I/O at socket. """
        if self.handler is not None:
            self.handler.handle_read()
    
    def handle_write(self):
        """ Handle write I/O at socket. """
        if self.handler is not None:
            self.handler.handle_write()
    
    def handle_error(self):
        """ Handle error in handler. """
        # This is the sanest default.
        if self.handler is not None:
            self.handler.handle_error()
    
    def handle_except(self, err):
        """ Handle exception state at error. """
        if self.handler is not None:
            self.handler.handle_except(err)
    
    def handle_connect(self):
        """ Connection established. """
        if self.handler is not None:
            self.handler.handle_connect()
    
    def handle_connect_failed(self, err):
        """ Connection couldn't be established. """
        if self.handler is not None:
            self.handler.handle_connect_failed(err)
    
    def handle_close(self):
        """ Connection closed. Note that this is only called if the
        connection is closed by the remote end! """
        # Ensure the handler is only called once, even if multiple connection
        # closed events should be fired (which could happen due to the code
        # in SocketTransport.send).
        if self.handler is not None and not self.closed:
            self.handler.handle_close()
        self.closed = True
    
    def handle_cleanup(self):
        """ Called whenever the Handler is voided, for whatever reason.
        This may be the shutdown of the program, the closing of the
        connection by the local end or the like. """
        if self.handler is not None:
            self.handler.handle_cleanup()


class SocketTransport(Transport):
    """ A transport that uses a socket to send and receive data. """
    def __init__(self, socket_map, sock=None, handler=None):
        super(SocketTransport, self).__init__(handler)
        # Make no assumptions on what we want to do with the handler.
        # The user will need to explicitely make it read- or writeable.
        if sock is None:
            sock = defaultsocket_factory()
        self._readable = False
        self._writeable = False
        
        self.awaiting_connect = False
        
        self.connected = False
             
        self.socket_map = socket_map
        self.socket = None
        self.set_socket(sock)
    
    def close(self):
        """ Close the socket. """
        self.socket_map.del_transport(self)
        self.connected = False
        try:
            self.socket.close()
        except socket.error, err:
            if err.args[0] not in (errno.ENOTCONN, errno.EBADF):
                raise
        finally:
            self.socket_map.notifier.cleanup_obj(self)
    
    def set_socket(self, sock):
        """ Set socket as the socket of the handler.
        If the socket is already connected do not call handle_connect
        anymore.
        
        The socket is automatically put into non-blocking mode.
        
        If the Handler already had a socket, remove it out of the SocketMap
        and add it with its new socket. """
        await = False
        if self.socket:
            # If we had a socket before, we are still in the SocketMap.
            # Remove us out of it.
            self.socket_map.del_transport(self)
        
        sock.setblocking(0)
        try:
            sock.getpeername()
        except socket.error, err:
            if err.args[0] == errno.ENOTCONN:
                await = True
                self.connected = False
            else:
                raise
        else:
            self.connected = True
            # To provide consistency. If an async socket that already had
            # .connect called on it, we couldn't tell whether handle_connect
            # will be called or not if we wouldn't call it here.
            self.handle_connect()
        
        self.socket = sock
        self.socket_map.add_transport(self)
        if await:
            self.await_connect()
    
    def get_readable(self):
        """ Check whether handler wants to read data. """
        return self._readable
    
    def set_readable(self, value):
        """ Set whether handler wants to read data. """
        if self._readable == value:
            # The state hasn't changed. Calling the SocketMap's handlers
            # again might confuse it.
            return
        
        self._readable = value
        if value:
            self.socket_map.add_reader(self)
        else:
            self.socket_map.del_reader(self)
    
    def get_writeable(self):
        """ Check whether handler wants to write data. """
        return self._writeable
    
    def set_writeable(self, value):
        """ Set whether handler wants to write data. """
        if self._writeable == value:
            # The state hasn't changed. Calling the SocketMap's handlers
            # again might confuse it.
            return
        
        self._writeable = value
        # If we are waiting for the connect write-event,
        # the handler is
        #   a) Already in the writers list if we want to add it to it.
        #   b) Needs to remain in the writers list if we wanted to
        #      remove it.
        if not self.awaiting_connect:
            if value:
                self.socket_map.add_writer(self)
            else:
                self.socket_map.del_writer(self)
    
    readable = property(get_readable, set_readable)
    writeable = property(get_writeable, set_writeable)
    
    def await_connect(self):
        """ Add handler to its socket-map's writers if necessary.
        
        At the next write-event from the socket-map handle_connect will
        be called. """
        # If we are already writeable, the handler is already in the
        # socket-map's writers.
        self.awaiting_connect = True
        if not self.writeable:
            self.socket_map.add_writer(self)
    
    def stop_awaiting_connect(self):
        """ Remove handler from its socket-map's writers if necessary.
       
        At the next write-event from the socket-map handle_connect will
        not be called. """
        # Because of the way E/PollSocketMap works, the flag has to be
        # set before del_writer is called!
        self.awaiting_connect = False
        # If we are writeable, the handler needs to remain in the
        # socket-map's writers.
        if not self.writeable:
            self.socket_map.del_writer(self)
    
    def fileno(self):
        """ Return fileno of underlying socket object.
        Needed for select.select. """
        return self.socket.fileno()
    
    def listen(self, num):
        """ Listen for a maximum of num connections. """
        if not self.readable:
            self.set_readable(True)
        return self.socket.listen(num)
    
    def bind(self, addr):
        """ Bind to address. """
        return self.socket.bind(addr)
    
    def accept(self):
        """ Accept incoming connection. Return (conn, addr). If either of
        them is None, no connection could be accepted. """
        try:
            conn, addr = self.socket.accept()
            return conn, addr
        except socket.error, err:
            if err.args[0] in trylater:
                # Make the API of returning a tuple of two objects consistent.
                return None, None
            else:
                raise
    
    def reuse_addr(self):
        """ Reuse the address. """
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    
    def send(self, data, flags=0):
        """ Send data.
    
        flags is constructed by ORing zero or more of the following values.
        
        MSG_OOB
            Sends out-of-band data on sockets that support this notion
            (e.g. SOCK_STREAM); the underlying protocol must also support
            out-of-band data.
        MSG_DONTROUTE
            Don't use a gateway to send out the packet, only send to hosts on
            directly connected networks. This is usually used only by diagnostic
            or routing programs. This is only defined for protocol families that
            route; packet sockets don't.
        MSG_DONTWAIT
            Enables non-blocking operation; if the operation would block,
            EAGAIN is returned.
        MSG_NOSIGNAL
            Requests not to send SIGPIPE on errors on stream oriented sockets
            when the other end breaks the connection. The EPIPE error is still
            returned.
        MSG_CONFIRM
            (Linux 2.3+ only) Tell the link layer that forward process happened:
            you got a successful reply from the other side. If the link layer
            doesn't get this it'll regularly reprobe the neighbour
            (e.g. via a unicast ARP). Only valid on SOCK_DGRAM and SOCK_RAW
            sockets and currently only implemented for IPv4 and IPv6.
            See arp(7) for details. """
        try:
            return self.socket.send(data)
        except socket.error, err:
            if err.args[0] in trylater:
                return 0
            elif err.args[0] == errno.EPIPE:
                self.socket_map.notifier.close_obj(self)
                return 0
            else:
                raise
    
    def recv(self, buffer_size, flags=0):
        """ Receive at most buffer_size bytes of data.
        
        flags may be constructed by ORing zero or more of the following values:
        MSG_PEEK
            Peeks at an incoming message.
            The data is treated as unread and the next recv() or similar
            function shall still return this data.
        MSG_OOB
            Requests out-of-band data.
            The significance and semantics of out-of-band data
            are protocol-specific.
        MSG_WAITALL
            On SOCK_STREAM sockets this requests that the function block
            until the full amount of data can be returned.
            The function may return the smaller amount of data if the
            socket is a message-based socket, if a signal is caught,
            if the connection is terminated, if MSG_PEEK was specified,
            or if an error is pending for the socket. """
        try:
            data = self.socket.recv(buffer_size, flags)
            if not data:
                self.socket_map.notifier.close_obj(self)
            return data
        except socket.error, err:
            if err.args[0] in connection_lost:
                self.socket_map.notifier.close_obj(self)
                return EMPTY_BYTES
            else:
                raise
    
    def peek(self, buffer_size):
        return self.recv(buffer_size, socket.MSG_PEEK)
    
    def connect(self, address):
        """ Connect to (host, port). """
        err = self.socket.connect_ex(address)
        # EWOULDBLOCK is only expected with WinSock.
        if err in (errno.EINPROGRESS, errno.EWOULDBLOCK):
            if not self.awaiting_connect:
                self.connected = False
                self.await_connect()
            return
        elif err == 0:
            if self.awaiting_connect:
                self.stop_awaiting_connect()
            self.connected = True
            self.handle_connect()
        else:
            # Used to be `raise socket.error(err, os.strerror(err))` but
            # it is a better idea to have all handling of connection
            # failed in the same place.
            try:
                self.handle_connect_failed(err)
            except Exception:
                self.handle_error()


class SendallTrait(object):
    """ Enhance Transport with sendall that buffers data and sends it all,
    if necessary in multiple steps. Mix into the transport to use.
    
        class SendAllTransport(asynchia.SendallTrait, asynchia.SocketTransport):
            pass
    """
    def __init__(self, *args, **kwargs):
        super(SendallTrait, self).__init__(*args, **kwargs)
        self.__buf = EMPTY_BYTES
        self.__savewriteable = None
    
    def set_writeable(self, value):
        """ If sendall is active, save the value passed and set it to that
        after sendall is finished. """
        if not self.__buf:
            super(SendallTrait, self).set_writeable(value)
        else:
            self.__savewriteable = value
    
    def get_writeable(self):
        """ Necessary for the property. """
        return super(SendallTrait, self).get_writeable()
    
    def sendall(self, data):
        """ Send all of data, if necessary in multiple steps. """
        if not self.__buf:
            self.__savewritable = self.writeable
        self.set_writeable(True)
        self.__buf += data
    
    def handle_write(self):
        """ Internal. """
        if not self.__buf:
            if self.__savewriteable is not None:
                super(SendallTrait, self).set_writeable(self.__savewriteable)
                self.__savewriteable = None
        
            if self.handler is not None and self.writeable:
                super(SendallTrait, self).handle_write()
        else:
            sent = super(SendallTrait, self).send(self.__buf)
            self.__buf = self.__buf[sent:]
    
    writeable = property(get_writeable, set_writeable)


class Handler(object):
    """ Handle a socket object. Call this objects handle_* methods upon I/O """
    # Dummy handlers.
    def __init__(self, transport):
        self.transport = transport
        
        self.transport.handler = self
    
    def handle_read(self):
        """ Handle read I/O at socket. """
        pass
    
    def handle_write(self):
        """ Handle write I/O at socket. """
        pass
    
    def handle_error(self):
        """ Handle error in handler. """
        # This is the sanest default.
        traceback.print_exc()
    
    def handle_except(self, err):
        """ Handle exception state at error. """
        pass
    
    def handle_connect(self):
        """ Connection established. """
        pass
    
    def handle_connect_failed(self, err):
        """ Connection couldn't be established. """
        pass
    
    def handle_close(self):
        """ Connection closed. Note that this is only called if the
        connection is closed by the remote end! """
        pass
    
    def handle_cleanup(self):
        """ Called whenever the Handler is voided, for whatever reason.
        This may be the shutdown of the program, the closing of the
        connection by the local end or the like. """


class AcceptHandler(Handler):
    """ Handle socket that accepts connections. """
    def __init__(self, transport=None):
        Handler.__init__(self, transport)
    
    def handle_read(self):
        """ Do not override. """
        sock, addr = self.transport.accept()
        if sock is not None:
            self.handle_accept(sock, addr)
    
    def handle_accept(self, sock, addr):
        """ Accept connection from addr at sock. """
        pass


class Server(AcceptHandler):
    """ Automatically create an instance of handlercls for every
    connection. """
    def __init__(self, transport, handlercls=Handler):
        AcceptHandler.__init__(self, transport)
        self.handlercls = handlercls
    
    def handle_accept(self, sock, addr):
        """ Instantiate the handler class. """
        handler = self.handlercls(
            SocketTransport(self.transport.socket_map, sock)
        )
        self.new_connection(handler, addr)
        return handler
    
    def new_connection(self, handler, addr):
        """ Called after a new handler has been created. """
    
    def serve_forever(self, addr, num):
        """ Serve a maximum of num connections at a time at addr. """
        self.transport.bind(addr)
        self.transport.listen(num)
        try:
            self.transport.socket_map.run()
        finally:
            self.transport.close()

defaultsocket_factory = socket.socket



#!/usr/bin/env python
# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

try:
    # If we got setuptools, use it so we get the nice develop cmd.
    from setuptools import setup
except:
    # Doesn't matter if we don't have it though.
    from distutils.core import setup


VERSION = '0.1.0a'


setup(
    name='asynchia',
    version=VERSION,
    description='asynchia is a minimalistic asynchronous networking library.',
    author='Florian Mayer',
    author_email='flormayer@aim.com',
    url='http://bitbucket.org/segfaulthunter/asynchia-mainline',
    keywords='async asynchronous network',
    license='LGPL',
    zip_safe=True,
    packages=['asynchia'],
)



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Auxiliary functions. """

import sys
import math
import socket
import collections

import asynchia.const

class GradualAverage(object):
    """ Memory-efficient average to which values may gradually be added
    over time. Its value can be accessed via the avg member. """
    def __init__(self, values=None):
        if values is None:
            self.avg = self.len = 0.0
        else:
            self.len = float(len(values))
            self.avg = sum(values) / self.len
    
    def add_value(self, value):
        """ Add value to the average. """
        self.avg = (self.avg * self.len + value) / (self.len + 1)
        self.len += 1
    
    def add_values(self, values):
        """ Add values to the average. This is more effective than using
        add_value multiple times. """
        self.avg = (
            self.avg * self.len + sum(values)
            ) / (self.len + len(values))
        self.len += len(values)


class LimitedAverage(object):
    """ Average considering the last samples values added to it. Its value
    can be accessed via the avg member. Stores up to samples objects in a
    deque and thus has a higher memory usage than GradualAverage (and a
    different use case). """
    def __init__(self, samples, values=None):
        self.cache = None
        
        self.samples = samples
        if values is None:
            self.values = collections.deque()
        else:
            self.values = collections.deque(values)
        
            for _ in xrange(len(values) - samples):
                self.values.popleft()
    
    def add_value(self, value):
        """ Add value to the average. """
        # Invalidate possibly existing cache.
        self.cache = None
        if len(self.values) == self.samples:
            self.values.popleft()
        self.values.append(value)
    
    def add_values(self, values):
        """ Add values to the average. This is more effective than using
        add_value multiple times. """
        # Invalidate possibly existing cache.
        self.cache = None
        self.values.extend(values)
        for _ in xrange(len(values) - self.samples):
            self.values.popleft()
    
    @property
    def avg(self):
        if self.cache is not None:
            return self.cache
        else:
            self.cache = sum(self.values) / float(len(self.values))
            return self.cache


class IDPool(object):
    """
    Pool that returns unique identifiers.
    
    Identifierers obtained using the get method are guaranteed to not be
    returned by it again until they are released using the release method.
    
        >>> pool = IDPool()
        >>> pool.get()
        0
        >>> pool.get()
        1
        >>> pool.get()
        2
        >>> pool.release(1)
        >>> pool.get()
        1
        >>> 
    """
    def __init__(self):
        self.max_id = -1
        self.free_ids = []
    
    def get(self):
        """ Return a new integer that is unique in this pool until
        it is released. """
        if self.free_ids:
            return self.free_ids.pop()
        else:
            self.max_id += 1
            return self.max_id
    
    def release(self, id_):
        """ Release the id. It can now be returned by get again.
        
        Will reset the IDPool if the last id in use is released. """
        self.free_ids.append(id_)
        if len(self.free_ids) == self.max_id + 1:
            self.reset()
    
    def reset(self):
        """ Reset the state of the IDPool. This should only be called when
        no identifier is in use. """
        self.max_id = -1
        self.free_ids = []


def socketpair():
    """ Return pair of connected sockets. Unlike socket.socketpair this
    is platform independant. However, if socket.socketpair is available,
    it is used here as well. """
    if hasattr(socket, 'socketpair'):
        # Unix.
        return socket.socketpair()
    
    try:
        acceptor = socket.socket()
        # Random port. Only accept local connections.
        acceptor.bind(('127.0.0.1', 0))
        # We know we'll only get one connection.
        acceptor.listen(1)

        one = socket.socket()
        one.connect(acceptor.getsockname())
        
        other = acceptor.accept()[0]
    finally:
        acceptor.close()
    return one, other


def parse_ipv4(string, default_port=-1):
    """ Return (host, port) from IPv4 IP. """
    split = string.split(':')
    if len(split) == 1:
        return string, default_port
    elif len(split) == 2:
        return split[0], int(split[1])
    else:
        raise ValueError("Cannot interpret %r as IPv4 address!" % string)


def parse_ipv6(string, default_port=-1):
    """ Return (host, port) from IPv6 IP. """
    if '[' in string and ']' in string:
        split = string.split(']:')
        if len(split) == 1:
            return string[1: -1], default_port
        else:
            return split[0][1:], int(split[1])
    elif not '[' in string and not ']' in string:
        return string, default_port
    else:
        raise ValueError("Cannot interpret %r as IPv6 address!" % string)


def parse_ip(string, default_port=-1):
    """ Return (host, port) from input string. This tries to automatically
    determine whether the address is IPv4 or IPv6.
    
    If no port is found default_port, which defaults to -1 is used.
    
    >>> parse_ip('127.0.0.1:1234')
    ('127.0.0.1', 1234)
    >>> parse_ip('[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:443')
    ('2001:0db8:85a3:08d3:1319:8a2e:0370:7344', 443)
    """
    if string.count(':') > 1:
        return parse_ipv6(string, default_port)
    else:
        return parse_ipv4(string, default_port)


def goodsize(maxsize):
    """ Return biggest power of two that is less or equal to maxsize.
    
    Powers of two are considered good values to be passed to the recv
    method of sockets. To quote the python documentation of the socket
    module:

        For best match with hardware and network realities,
        the value of bufsize should be a relatively small power of 2,
        for example, 4096.
"""
    return 2 ** math.floor(math.log(maxsize, 2))


def is_closed(sock):
    """ Find out whether a socked is closed or not.
    Does only work on non-blocking sockets! """
    try:
        rcv = sock.recv(1, socket.MSG_PEEK)
    except socket.error, err:
        if err.args[0] in asynchia.const.trylater:
            return False
        else:
            raise
    else:
        return not rcv


if sys.version_info >= (3, 0):
    def b(s):
        return s.encode('utf-8')
else:
    b = str


EMPTY_BYTES = b('')

if sys.version_info >= (3, 0):
    def byte_std(stream):
        return stream.buffer
else:
    def byte_std(stream):
        return stream



# -*- coding: us-ascii -*-

# asynchia - asynchronous networking library
# Copyright (C) 2009 Florian Mayer

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" Commonly used protocols. """

import asynchia
from asynchia.util import EMPTY_BYTES

class LineHandler(asynchia.Handler):
    """ Use this for line-based protocols. """
    delimiter = None
    buffer_size = 4096
    def __init__(self, transport):
        asynchia.Handler.__init__(self, transport)
        self.read_buffer = EMPTY_BYTES
        if not self.transport.readable:
            self.transport.set_readable(True)
    
    def split_buffer(self):
        """ Split buffer into the different lines. """
        split = self.read_buffer.split(self.delimiter)
        self.read_buffer = split.pop()
        return split
    
    def parse_buffer(self):
        """ Call the line_received method for any lines delimited by
        self.delimited that are in the buffer. """
        for line in self.split_buffer():
            self.line_received(line)
    
    def handle_read(self):
        """ We got inbound data. Extend our buffer and see if we have
        got lines in it. """
        self.read_buffer += self.transport.recv(self.buffer_size)
        self.parse_buffer()
    
    def send_line(self, line):
        """ Send line. To use this SendallTrait must be mixed into
        your transport! """
        self.transport.sendall(line + self.delimiter)
    
    def line_received(self, line):
        """ Override. """
        pass
